% This program computes a first-order approximation to the costate functions
% VK(K,A;eta) and VA(K,A;eta) of the simple stochastic growth model in Section 3.3 in 
% ``Risk Matters: Breaking Certainty Equivalence in Linear Approximations,''
% by Juan Carlos Parra-Alvarez, Hamza Polattimur and Olaf Posch. 
%
% The equilibrium conditions of the model can be written as: H(x,y,y_x,y_xx)=0, 
% The solution is of the form
% y = g(x,eta), where x is the nx vector of state variables, and eta is theperturbation parameter.
% The risk-sensitive first-order approximation to this function for control i = 1,..., nx is
% y^i = gss^i + gx^i_a (x_a - xss_a) + geta^i eta
% where: 
% gss  is nx by nx
% gx   is nx by nx
% geta is nx by 1 
%
% User inputs:
% Input 1: Parameter values 
% Input 2: Symbolic definition of control and state variables
% Input 3: Coefficients a (nx by 1), b (nx by 1) and c (nx^2 by 1) of the system of quasilinear PDEs
% Input 4: Deterministic steady state (DSS)
%
% Using the approximations to VK and VA, the program also reports the associated
% linear approximation to the optimal consumption function, u = u(x,y).
%
% (c) Juan Carlos Parra-Alvarez, Hamza Polattimur and Olaf Posch
% Date May 25, 2021.

%% =======================================================================%
%                             Housekeeping                                %
%-------------------------------------------------------------------------%
clear all
clc
close all


%% =======================================================================%
%                                 Input 1:                                %
%                         Structural parameters                           %
%-------------------------------------------------------------------------%
aalpha  = 0.36;
ddelta  = 0.0963;
rrho    = 0.0410;
ggamma  = 2;
rrhoA   = 0.2052;
ssigmaA = 0.0307;

%% =======================================================================%
%                                 Input 2:                                %
%                       Define control and states (y,x)                   %
%-------------------------------------------------------------------------%
syms A K VK VA;
y  = [VK, VA];
x  = [K, A];
ny = length(y);
nx = length(x);
n  = ny + nx;

%% =======================================================================%
%                                 Input 3:                                %
%                Define PDE coefficient a, b and c (Eq. 20)               %
%-------------------------------------------------------------------------%
a = [(aalpha*exp(A)*K^(aalpha-1) - ddelta)*VK; 
    exp(A)*K^(aalpha)*VK - rrhoA*VA];

b = [exp(A)*K^(aalpha) - VK^(-1/ggamma) - ddelta*K;
    -rrhoA*A];

c = vec([0,0;0,1/2*ssigmaA^2]);

%% =======================================================================%
%                                 Input 4:                                %
%                     Deterministic steady state (DSS)                    %
%-------------------------------------------------------------------------%
Ass  = 0;
Kss  = (aalpha/(rrho+ddelta))^(1/(1-aalpha));
VKss = (Kss^aalpha - ddelta*Kss)^(-ggamma);
VAss = Kss^aalpha*VKss/(rrho+rrhoA);
ss   = [VKss; VAss; Kss; Ass];
gss  = ss(1:ny);
xss  = ss(ny+1:n);


%% =======================================================================%
%       Compute first order derivatives of PDE coefficient a, b and c     %
%-------------------------------------------------------------------------%
% First order terms
by = jacobian(b,y);
bx = jacobian(b,x);
ay = jacobian(a,y);
ax = jacobian(a,x);

% First order terms at DSS
byss = double(subs(by,[y,x],[gss',xss']));
bxss = double(subs(bx,[y,x],[gss',xss']));
ayss = double(subs(ay,[y,x],[gss',xss']));
axss = double(subs(ax,[y,x],[gss',xss']));

%% =======================================================================%
%                             Proposition 1                               %
%                         Ricatti equation (Eq. 28)                       %
%                      A' gx + gx A + gx C gx + B = 0                     %
%-------------------------------------------------------------------------%
A = bxss -rrho/2*eye(nx);
C = byss;
B = axss;

% Hamiltonian matrix (Eq. 29)
H     = [A, C;-B, -A'];

% Jordan decomposition (Eq. 30)
[P,L] = eig(H'); % P eigevectors, L eigenvalues
temp  = sortrows([diag(L) P'],1); % sort in ascending order by eigenvalues
L     = diag(temp(:,1));
P     = temp(:,2:end);

% Stability check
sH = sign(diag(L));
if sum(sH(:)==-1) == nx
    fprintf('---------------------------------------- \n');
    fprintf('Blanchard and Kahn conditions satisfied  \n');
    fprintf('---------------------------------------- \n');
else
    fprintf('----------------------------------------------------------------------------- \n');
    fprintf('Error: Number of stable eigevalues is different to number of state variables  \n');
    fprintf('----------------------------------------------------------------------------- \n');
    return;
end

%% =======================================================================%
%                            Solution gx (Eq. 31)                         %
%-------------------------------------------------------------------------%
gx = -inv(P(nx+1:n,nx+1:n))*P(nx+1:n,1:nx);

%% =======================================================================%
%      Compute second order derivatives of PDE coefficient a, b and c     %
%-------------------------------------------------------------------------%
% Second order terms
axx = reshape(jacobian(ax(:),x),ny,nx,nx);
bxx = reshape(jacobian(bx(:),x),nx,nx,nx);
ayx = reshape(jacobian(ay(:),x),ny,ny,nx); 
axy = reshape(jacobian(ax(:),y),ny,nx,ny); 
byx = reshape(jacobian(by(:),x),nx,ny,nx);
bxy = reshape(jacobian(bx(:),y),nx,ny,nx); 
ayy = reshape(jacobian(ay(:),y),ny,ny,ny); 
byy = reshape(jacobian(by(:),y),nx,ny,ny);

% Second order terms at DSS
axxss = double(subs(axx,[y,x],[gss',xss']));
bxxss = double(subs(bxx,[y,x],[gss',xss']));
ayxss = double(subs(ayx,[y,x],[gss',xss']));
axyss = double(subs(axy,[y,x],[gss',xss']));
byxss = double(subs(byx,[y,x],[gss',xss']));
bxyss = double(subs(bxy,[y,x],[gss',xss']));
ayyss = double(subs(ayy,[y,x],[gss',xss']));
byyss = double(subs(byy,[y,x],[gss',xss']));

% Reshaping second order terms
axxss = reshape(axxss,ny,nx^2);
bxxss = reshape(bxxss,nx,nx^2);
ayxss = reshape(ayxss,ny,ny*nx);
axyss = reshape(axyss,ny,nx*ny);
byxss = reshape(byxss,nx,ny*nx);
bxyss = reshape(bxyss,nx,nx*ny);
ayyss = reshape(ayyss,ny,ny^2);
byyss = reshape(byyss,nx,ny^2);

% Commutation matrix
K = zeros(nx^2,nx^2);
for i = 1:nx
  for j = 1:nx
    K(i + nx*(j - 1), j + nx*(i - 1)) = 1;
  end
end

%% =======================================================================%
%                             Proposition 3                               %
%                       Sylvester equation (Eq. 33)                       %
%                           R gxx + gxx Q + S = 0                         %
%-------------------------------------------------------------------------%
R = ayss - rrho*eye(nx) + gx*byss;
Q = kron(eye(nx),bxss) + kron(eye(nx),byss)*kron(eye(nx),gx) + kron(eye(nx),(bxss+byss*gx))*K; 
S = axxss + axyss*kron(gx,eye(nx)) + (ayxss+ayyss*kron(gx,eye(ny)))*kron(eye(nx),gx) ...
    + gx*(bxxss+bxyss*kron(gx,eye(nx))) + gx*(byxss+byyss*kron(gx,eye(ny)))*kron(eye(nx),gx);

M = kron(eye(nx^2),R) + kron(Q',eye(ny));
L = vec(S);

%% =======================================================================%
%                            Solution gxx (Eq. 34)                        %
%-------------------------------------------------------------------------%
v_gxx = -M\L;
gxx   = reshape(v_gxx,ny,nx^2);

%% =======================================================================%
%                                Theorem 2                                %
%                           Solution geta (Eq. 32)                        %
%-------------------------------------------------------------------------%
geta = -inv(ayss-rrho*eye(nx)+gx*byss)*(gxx*c);

%% =======================================================================%
%                             Print results                               %
%-------------------------------------------------------------------------%
disp(' ')
disp('Coefficients linear approximation to costates, y = g(x,eta) ')
disp(' ')
disp('g(xss;0)')
gss
disp(' ')

disp(' ')
disp('gx(xss;0)')
gx
disp(' ')

disp(' ')
disp('geta(xss;0)')
geta
disp(' ')

%% =======================================================================%
%                       Optimal control, u = u(x,y)                       %
%-------------------------------------------------------------------------%
CK   = -(1/ggamma)*gss(1)^(-1/ggamma-1)*gx(1,1);
CA   = -(1/ggamma)*gss(1)^(-1/ggamma-1)*gx(1,2);
Ceta = -(1/ggamma)*gss(1)^(-1/ggamma-1)*geta(1,1);
Cx   = [CK, CA];
Css  = VKss^(-1/ggamma);

disp(' ')
disp('Coefficients linear approximation to consumption, c = c(x,eta)')
disp(' ')
Css
disp(' ')
Cx
disp(' ')
Ceta
disp(' ')



